-- Counter: Library module created by VHDL Module Generator
--
-- Counter
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;

entity Counter is
	generic (
		CntWidth: integer := 4;
		InitialValue: integer := 0;
		EndValue: integer := EndValue;
		Increment: integer := 1;
		StartActive: STD_LOGIC := '1';
		EnableActive: STD_LOGIC := '1';
		DataType: type := UNSIGNED 
	);
	port (
		START,EN: in STD_LOGIC;
		Cnt: out DataType(CntWidth-1 downto 0);
		CLK: in STD_LOGIC
	);
	type ENUM_Style is (RunStop, Cyclic);
	constant Style: ENUM_Style := Cyclic;
end Counter;

architecture Binary of Counter is
	signal tCnt: DataType(CntWidth-1 downto 0);
begin

	process (CLK)
	begin
		if (CLK'event and CLK = '1') then
			if START = StartActive then
				tCnt <= CONV_DataType(InitialValue,CntWidth);
			elsif EN=EnableActive then
				if tCnt=EndValue and Style=Cyclic and InitialValue/=EndValue then
					tCnt <= CONV_DataType(InitialValue,CntWidth);
				elsif tCnt /= EndValue or EndValue=InitialValue then
					tCnt <= tCnt + CONV_DataType(Increment,CntWidth);
				end if;
			end if;
		end if;
	end process;

	Cnt <= tCnt;

end Binary;

architecture One_Hot of Counter is
	signal tCnt: DataType(CntWidth-1 downto 0);
begin

	process (CLK)
	begin
		if (CLK'event and CLK='1') then
			if (START = StartActive) then
				tCnt <= (others=>'0');
				tCnt(InitialValue) <= '1';
			elsif EN=EnableActive then
				if tCnt(EndValue)='1' and Style=Cyclic and InitialValue/=EndValue then
					tCnt <= (others=>'0');
					tCnt(InitialValue) <= '1';
				elsif tCnt(EndValue)='0' or EndValue=InitialValue then
					tCnt <= tCnt(CntWidth-Increment-1 downto 0) & tCnt(CntWidth-1 downto CntWidth-Increment);
				end if;
			end if;
		end if;
	end process;

	Cnt <= tCnt;

end One_Hot;

architecture Johnson of Counter is
	signal tCnt: DataType(CntWidth-1 downto 0);
begin

	process (CLK)
	begin
		if (CLK'event and CLK='1') then
			if (START = StartActive) then
				tCnt <= CONV_DataType(InitialValue,CntWidth);
			elsif EN=EnableActive then
				if tCnt=EndValue and Style=Cyclic and InitialValue/=EndValue then
					tCnt <= CONV_DataType(InitialValue,CntWidth);
				elsif tCnt /= EndValue or EndValue=InitialValue then
					tCnt <= tCnt(CntWidth-2 downto 0) & not(tCnt(CntWidth-1));
				end if;
			end if;
		end if;
	end process;

	Cnt <= tCnt;

end Johnson;

-- Counter: Library module ended here

