-- alarmclock
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;

entity alarmclock is
	port (
		START : in STD_LOGIC;
		EN : in STD_LOGIC;
		PRESETALARM : in UNSIGNED(15 downto 0);
		CLK : in STD_LOGIC;
		LEDm1 : out STD_LOGIC_VECTOR(6 downto 0);
		LEDm10 : out STD_LOGIC_VECTOR(6 downto 0);
		LEDh1 : out STD_LOGIC_VECTOR(6 downto 0);
		LEDh10 : out STD_LOGIC_VECTOR(6 downto 0);
		ALARM : out STD_LOGIC
	);
end alarmclock;

architecture alarmclock_arch of alarmclock is
	signal HEXm1,HEXm10,HEXh1,HEXh10 : UNSIGNED(3 downto 0);
	signal BCDhhmm : UNSIGNED(15 downto 0);

	-- VHDL Module Generator component declarations
	component clock
	port (
		START : in STD_LOGIC;
		EN : in STD_LOGIC;
		LEDm1 : out STD_LOGIC_VECTOR(6 downto 0);
		LEDm10 : out STD_LOGIC_VECTOR(6 downto 0);
		LEDh1 : out STD_LOGIC_VECTOR(6 downto 0);
		LEDh10 : out STD_LOGIC_VECTOR(6 downto 0);
		HEXm1 : out UNSIGNED(3 downto 0);
		HEXm10 : out UNSIGNED(3 downto 0);
		HEXh1 : out UNSIGNED(3 downto 0);
		HEXh10 : out UNSIGNED(3 downto 0);
		CLK : in STD_LOGIC
	);
	end component;

	component Comparator
	generic (
		EQUAL: note := "EQUAL='1' when A = B";
		GREATER: note := "GREATER='1' when A > B";
		SMALLER: note := "SMALLER='1' when A < B";
		AWidth: integer := 16;
		BWidth: integer := 16;
		InType: type := UNSIGNED 
	);
	port (
		A: in InType(AWidth-1 downto 0);
		B: in InType(BWidth-1 downto 0);
		EQUAL,GREATER,SMALLER : out STD_LOGIC
	);
	end component;

	for Conf_Comparator: Comparator
		use entity WORK.Comparator(SMALL);

begin

	U_clock: clock
		port map (START, EN, LEDm1, LEDm10, LEDh1, LEDh10, HEXm1, HEXm10, HEXh1, HEXh10, CLK);

	BCDhhmm <= HEXh10 & HEXh1 & HEXm10 & HEXm1;

	U_Comparator: Comparator
		port map (A=>BCDhhmm, B=>PRESETALARM, EQUAL=>ALARM);

end alarmclock_arch;

