-- digit
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;

entity digit is
	generic (
		Note: note := "1-digit counter with 7-segment display output";
		EndValue: integer := 9 
	);
	port (
		START : in STD_LOGIC;
		EN : in STD_LOGIC;
		HEX : out UNSIGNED(3 downto 0);
		LED : out STD_LOGIC_VECTOR(6 downto 0);
		CLK : in STD_LOGIC
	);
end digit;

architecture digit_arch of digit is

	signal HEXint : UNSIGNED(3 downto 0);

	-- VHDL Module Generator component declarations
	component Hex2LED
	port (
		HEX : in UNSIGNED(3 downto 0);
		LED : out STD_LOGIC_VECTOR(6 downto 0)
	);
	end component;

	component Counter
	generic (
		CntWidth: integer := 4;
		InitialValue: integer := 0;
		EndValue: integer := EndValue;
		Increment: integer := 1;
		StartActive: STD_LOGIC := '1';
		EnableActive: STD_LOGIC := '1';
		DataType: type := UNSIGNED 
	);
	port (
		START,EN: in STD_LOGIC;
		Cnt: out DataType(CntWidth-1 downto 0);
		CLK: in STD_LOGIC
	);
	end component;

	for Conf_Counter: Counter
		use entity WORK.Counter(Binary);

begin
	U_Counter: Counter
		port map (START, EN, HEXint, CLK);

	U_Hex2LED: Hex2LED
		port map (HEXint, LED);

	HEX <= HEXint;

end digit_arch;

