-- digit: Library module created by VHDL Module Generator
--
-- digit
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;

entity digit_s10 is
	generic (
		EndValue: integer := 5 
	);
	port (
		START : in STD_LOGIC;
		EN : in STD_LOGIC;
		HEX : out UNSIGNED(3 downto 0);
		LED : out STD_LOGIC_VECTOR(6 downto 0);
		CLK : in STD_LOGIC
	);
end digit_s10;

architecture digit_s10_arch of digit_s10 is

	signal HEXint : UNSIGNED(3 downto 0);

	-- VHDL Module Generator component declarations
	component Hex2LED
	port (
		HEX : in UNSIGNED(3 downto 0);
		LED : out STD_LOGIC_VECTOR(6 downto 0)
	);
	end component;

	component Counter
	generic (
		CntWidth: integer := 4;
		InitialValue: integer := 0;
		EndValue: integer := EndValue;
		Increment: integer := 1;
		StartActive: STD_LOGIC := '1';
		EnableActive: STD_LOGIC := '1';
		DataType: type := UNSIGNED 
	);
	port (
		START,EN: in STD_LOGIC;
		Cnt: out DataType(CntWidth-1 downto 0);
		CLK: in STD_LOGIC
	);
	end component;

	for Conf_Counter: Counter
		use entity WORK.Counter(Binary);

begin
	U_Counter: Counter
		port map (START, EN, HEXint, CLK);

	U_Hex2LED: Hex2LED
		port map (HEXint, LED);

	HEX <= HEXint;

end digit_s10_arch;

-- Hex2LED
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;

entity Hex2LED is
	port (
		HEX : in UNSIGNED(3 downto 0);
		LED : out STD_LOGIC_VECTOR(6 downto 0)
	);
end Hex2LED;

architecture Hex2LED_arch of Hex2LED is
begin

	with HEX select LED <=
		"1111001" when "0001",	--1
		"0100100" when "0010",	--2
		"0110000" when "0011",	--3
		"0011001" when "0100",	--4
		"0010010" when "0101",	--5
		"0000010" when "0110",	--6
		"1111000" when "0111",	--7
		"0000000" when "1000",	--8
		"0010000" when "1001",	--9
		"0001000" when "1010",	--A
		"0000011" when "1011",	--b
		"1000110" when "1100",	--C
		"0100001" when "1101",	--d
		"0000110" when "1110",	--E
		"0001110" when "1111",	--F
		"1000000" when others;	--0

end Hex2LED_arch;

-- Counter: Library module created by VHDL Module Generator
--
-- Counter
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;

entity Counter is
	generic (
		CntWidth: integer := 4;
		InitialValue: integer := 0;
		EndValue: integer := EndValue;
		Increment: integer := 1;
		StartActive: STD_LOGIC := '1';
		EnableActive: STD_LOGIC := '1';
		DataType: type := UNSIGNED 
	);
	port (
		START,EN: in STD_LOGIC;
		Cnt: out DataType(CntWidth-1 downto 0);
		CLK: in STD_LOGIC
	);
	type ENUM_Style is (RunStop, Cyclic);
	constant Style: ENUM_Style := Cyclic;
end Counter;

architecture Binary of Counter is
	signal tCnt: DataType(CntWidth-1 downto 0);
begin

	process (CLK)
	begin
		if (CLK'event and CLK = '1') then
			if START = StartActive then
				tCnt <= CONV_DataType(InitialValue,CntWidth);
			elsif EN=EnableActive then
				if tCnt=EndValue and Style=Cyclic and InitialValue/=EndValue then
					tCnt <= CONV_DataType(InitialValue,CntWidth);
				elsif tCnt /= EndValue or EndValue=InitialValue then
					tCnt <= tCnt + CONV_DataType(Increment,CntWidth);
				end if;
			end if;
		end if;
	end process;

	Cnt <= tCnt;

end Binary;

architecture One_Hot of Counter is
	signal tCnt: DataType(CntWidth-1 downto 0);
begin

	process (CLK)
	begin
		if (CLK'event and CLK='1') then
			if (START = StartActive) then
				tCnt <= (others=>'0');
				tCnt(InitialValue) <= '1';
			elsif EN=EnableActive then
				if tCnt(EndValue)='1' and Style=Cyclic and InitialValue/=EndValue then
					tCnt <= (others=>'0');
					tCnt(InitialValue) <= '1';
				elsif tCnt(EndValue)='0' or EndValue=InitialValue then
					tCnt <= tCnt(CntWidth-Increment-1 downto 0) & tCnt(CntWidth-1 downto CntWidth-Increment);
				end if;
			end if;
		end if;
	end process;

	Cnt <= tCnt;

end One_Hot;

architecture Johnson of Counter is
	signal tCnt: DataType(CntWidth-1 downto 0);
begin

	process (CLK)
	begin
		if (CLK'event and CLK='1') then
			if (START = StartActive) then
				tCnt <= CONV_DataType(InitialValue,CntWidth);
			elsif EN=EnableActive then
				if tCnt=EndValue and Style=Cyclic and InitialValue/=EndValue then
					tCnt <= CONV_DataType(InitialValue,CntWidth);
				elsif tCnt /= EndValue or EndValue=InitialValue then
					tCnt <= tCnt(CntWidth-2 downto 0) & not(tCnt(CntWidth-1));
				end if;
			end if;
		end if;
	end process;

	Cnt <= tCnt;

end Johnson;

-- Counter: Library module ended here

-- digit: Library module ended here

